from sys import platform
from datetime import datetime as dt

import praw
from slacker import Slacker

from recho import __version__ as recho_version
from recho.reddit import RedditComment, RedditSubmission


def build_user_agent():
    recho_version_trunk = recho_version.split("+")[0]
    user_agent = '{0}:recho:{1} (by /u/IHKAS1984)'
    user_agent = user_agent.format(platform, recho_version_trunk)

    return user_agent


def get_reddit_posts_since(redditor_name, timestamp):
    """ Yields all reddit posts since timestamp for redditor
    """
    reddit = praw.Reddit(user_agent=build_user_agent())
    redditor = reddit.get_redditor(redditor_name)
    activity = list()

    # Get comment activity
    for comment in redditor.get_comments(time='week'):
        if timestamp >= dt.utcfromtimestamp(comment.created_utc):
            break

        activity.append(RedditComment(comment))

    # Get submitted post activity
    for submission in redditor.get_submitted(time='week'):
        if timestamp >= dt.utcfromtimestamp(submission.created_utc):
            break

        activity.append(RedditSubmission(submission))

    return sorted(activity, key=lambda x: x.created)


def post_to_slack(slack_config, posts):
    """ Post the given post or comment to the configured slack channel

        returns timestamp of the last posted message
    """
    slack = Slacker(slack_config['token'])

    last_timestamp = None
    for post in posts:
        message = format_for_slack(post)
        slack.chat.post_message(slack_config['channel'], message, as_user=True)
        last_timestamp = dt.utcfromtimestamp(post.created_utc)

    return last_timestamp


def format_for_slack(comment):
    """ Format a reddit comment for slack
    """
    slack_post = ("*User*:          {user_name}\n"
                  "*Thread*:      {thread}\n"
                  "*Permalink*: `{permalink}`\n"
                  "{text}")

    user_name = comment.author  # pylint: disable=W0612
    thread = comment.title[:75]  # pylint: disable=W0612
    permalink = comment.permalink  # pylint: disable=W0612
    text = comment.text  # pylint: disable=W0612

    return slack_post.format(**locals())



class UnsupportedSubmissionError(Exception):
    pass


class RedditBase(object):  # pylint: disable=R0903
    """ Base model for Reddit Post and Comment objects from Praw
    """
    def __init__(self, obj):
        self._obj = obj

    def __getattr__(self, name):
        return getattr(self._obj, name)

    def __str__(self):
        return str(self)

    def __repr__(self):
        return "{0} -> {1}".format(self._obj.id, str(self))


class RedditComment(RedditBase):
    """ Object model for Reddit comments
    """
    @property
    def text(self):
        """ Returns the full body of the comment
        """
        return "\n>>>" + self._obj.body

    @property
    def title(self):
        """ Returns the title of the comment, truncated to 75 characters
        """
        return self._obj.link_title[:75]


class RedditSubmission(RedditBase):
    """ Object model for Reddit submissions
    """
    @property
    def text(self):
        """ Returns the text of the
        """
        if hasattr(self._obj, 'selftext'):
            text = self._obj.selftext if self._obj.selftext else " <no text>"
            return ">>> " + text
        elif hasattr(self._obj, 'preview') and 'images' in self._obj.preview:
            return self._obj.preview['images'][0]['source']['url']
        else:
            exc_msg = "Found unsupported submission type: {}".format(self._obj.id)
            raise UnsupportedSubmissionError(exc_msg)

    @property
    def title(self):
        """ Returns the title of the comment, truncated to 75 characters
        """
        return self._obj.title[:75]



import os
import json
import argparse
from datetime import datetime as dt

from raven import Client
from configparser import ConfigParser

from recho import __version__ as recho_version
from recho.recho import get_reddit_posts_since, post_to_slack

CONFIG_NAME = '.recho.ini'

TS_NAME = '.recho_timestamps'
TS_FILEPATH = os.path.join(os.path.dirname(__file__), TS_NAME)


class RechoError(Exception):
    pass


def _get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'redditor', type=str,
        help="The name of the target Redditor"
    )

    return parser.parse_args()


def _load_config():
    """ Load the configuration file """
    config = ConfigParser()
    config_path = os.path.join(os.path.expanduser('~'), CONFIG_NAME)

    if not os.path.exists(config_path):
        raise RechoError("Config file must be present in ~/.recho.ini")

    config.read(config_path)

    return config


def _get_last_seen(redditor):
    """ Return the last time a check was done """
    if not os.path.exists(TS_FILEPATH):
        with open(TS_FILEPATH, 'w') as w:  # pylint: disable=C0103
            w.write(json.dumps({}))

    with open(TS_FILEPATH, 'r') as r:
        timestamps = json.load(r)

    if redditor not in timestamps:
        last_seen = dt.utcnow()
        timestamps[redditor] = last_seen.timestamp()
        with open(TS_FILEPATH, 'w') as w:  # pylint: disable=C0103
            w.write(json.dumps(timestamps, indent=4, sort_keys=True))
    else:
        last_seen = dt.fromtimestamp(float(timestamps[redditor]))

    return last_seen


def main():
    # Load CLI args
    args = _get_args()

    # Load config
    config = _load_config()

    try:
        # Get last timestamp
        last_seen = _get_last_seen(args.redditor)

        # Get new comments and threads from reddit
        new_posts = get_reddit_posts_since(args.redditor, last_seen)

        if new_posts:
            # Post new comments and threads to slack
            latest_timestamp = post_to_slack(config['slack'], new_posts)

            # Write new timestamp
            with open(TS_FILEPATH, 'r') as r:  # pylint: disable=C0103
                timestamps = json.load(r)

            timestamps[args.redditor] = latest_timestamp.timestamp()
            with open(TS_FILEPATH, 'w') as w:  # pylint: disable=C0103
                w.write(json.dumps(timestamps, indent=4, sort_keys=True))
    except:
        # Log to sentry, if configured
        if 'sentry' in config:
            key, secret = config['sentry']['key'], config['sentry']['secret']
            url = 'https://{0}:{1}@sentry.io/173123'.format(key, secret)
            Client(url, release=recho_version).captureException()

        raise
